@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2012  BMW Car IT GbmH. All rights reserved.
+ *  Copyright (C) 2012-2014  BMW Car IT GmbH.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -459,6 +459,8 @@ static int load_policy(GKeyFile *keyfile, const char *groupname,
 	str = g_key_file_get_string(keyfile, groupname, "AllowedBearers",
 				NULL);
 	if (str) {
+		g_slist_free(config->allowed_bearers);
+		config->allowed_bearers = NULL;
 		tokens = g_strsplit(str, " ", 0);
 
 		for (i = 0; tokens[i]; i++) {
@@ -565,7 +567,9 @@ static void recheck_sessions(void)
 		if (policy->group)
 			continue;
 
-		group = g_hash_table_lookup(selinux_hash, policy->selinux);
+		if (policy->selinux)
+			group = g_hash_table_lookup(selinux_hash,
+							policy->selinux);
 		if (group) {
 			policy->config->id_type = CONNMAN_SESSION_ID_TYPE_LSM;
 			g_free(policy->config->id);
@@ -621,7 +625,7 @@ static int load_file(const char *filename, struct policy_file *file)
 
 	for (i = 0; groupnames[i]; i++) {
 		group = g_new0(struct policy_group, 1);
-		group->config = g_new0(struct connman_session_config, 1);
+		group->config = connman_session_create_default_config();
 
 		err = load_policy(keyfile, groupnames[i], group);
 		if (err < 0) {
