@@ -215,6 +215,7 @@ static GSList *request_list = NULL;
 static GHashTable *listener_table = NULL;
 static time_t next_refresh;
 static GHashTable *partial_tcp_req_table;
+static guint cache_timer = 0;
 
 static guint16 get_id(void)
 {
@@ -332,14 +333,14 @@ static void refresh_dns_entry(struct cache_entry *entry, char *name)
 	}
 
 	if (!entry->ipv4) {
-		DBG("Refresing A record for %s", name);
+		DBG("Refreshing A record for %s", name);
 		g_resolv_lookup_hostname(ipv4_resolve, name,
 					dummy_resolve_func, NULL);
 		age = 4;
 	}
 
 	if (!entry->ipv6) {
-		DBG("Refresing AAAA record for %s", name);
+		DBG("Refreshing AAAA record for %s", name);
 		g_resolv_lookup_hostname(ipv6_resolve, name,
 					dummy_resolve_func, NULL);
 		age = 4;
@@ -530,6 +531,8 @@ static void destroy_request_data(struct request_data *req)
 static gboolean request_timeout(gpointer user_data)
 {
 	struct request_data *req = user_data;
+	struct sockaddr *sa;
+	int sk;
 
 	if (!req)
 		return FALSE;
@@ -537,47 +540,38 @@ static gboolean request_timeout(gpointer user_data)
 	DBG("id 0x%04x", req->srcid);
 
 	request_list = g_slist_remove(request_list, req);
-	req->numserv--;
+
+	if (req->protocol == IPPROTO_UDP) {
+		sk = get_req_udp_socket(req);
+		sa = &req->sa;
+	} else if (req->protocol == IPPROTO_TCP) {
+		sk = req->client_sk;
+		sa = NULL;
+	} else
+		goto out;
 
 	if (req->resplen > 0 && req->resp) {
-		int sk, err;
+		/*
+		 * Here we have received at least one reply (probably telling
+		 * "not found" result), so send that back to client instead
+		 * of more fatal server failed error.
+		 */
+		if (sk >= 0)
+			sendto(sk, req->resp, req->resplen, MSG_NOSIGNAL,
+				sa, req->sa_len);
 
-		if (req->protocol == IPPROTO_UDP) {
-			sk = get_req_udp_socket(req);
-			if (sk < 0)
-				return FALSE;
-
-			err = sendto(sk, req->resp, req->resplen, MSG_NOSIGNAL,
-				&req->sa, req->sa_len);
-		} else {
-			sk = req->client_sk;
-			err = send(sk, req->resp, req->resplen, MSG_NOSIGNAL);
-			if (err < 0)
-				close(sk);
-		}
-		if (err < 0)
-			return FALSE;
-	} else if (req->request && req->numserv == 0) {
+	} else if (req->request) {
+		/*
+		 * There was not reply from server at all.
+		 */
 		struct domain_hdr *hdr;
 
-		if (req->protocol == IPPROTO_TCP) {
-			hdr = (void *) (req->request + 2);
-			hdr->id = req->srcid;
-			send_response(req->client_sk, req->request,
-				req->request_len, NULL, 0, IPPROTO_TCP);
+		hdr = (void *)(req->request + protocol_offset(req->protocol));
+		hdr->id = req->srcid;
 
-		} else if (req->protocol == IPPROTO_UDP) {
-			int sk;
-
-			hdr = (void *) (req->request);
-			hdr->id = req->srcid;
-
-			sk = get_req_udp_socket(req);
-			if (sk >= 0)
-				send_response(sk, req->request,
-					req->request_len, &req->sa,
-					req->sa_len, IPPROTO_UDP);
-		}
+		if (sk >= 0)
+			send_response(sk, req->request, req->request_len,
+				sa, req->sa_len, req->protocol);
 	}
 
 	/*
@@ -590,6 +584,7 @@ static gboolean request_timeout(gpointer user_data)
 				GINT_TO_POINTER(req->client_sk));
 	}
 
+out:
 	req->timeout = 0;
 	destroy_request_data(req);
 
@@ -768,6 +763,8 @@ static void cache_element_destroy(gpointer value)
 
 static gboolean try_remove_cache(gpointer user_data)
 {
+	cache_timer = 0;
+
 	if (__sync_fetch_and_sub(&cache_refcount, 1) == 1) {
 		DBG("No cache users, removing it.");
 
@@ -2208,8 +2205,8 @@ static void destroy_server(struct server_data *server)
 	 * without any good reason. The small delay allows the new RDNSS to
 	 * create a new DNS server instance and the refcount does not go to 0.
 	 */
-	if (cache)
-		g_timeout_add_seconds(3, try_remove_cache, NULL);
+	if (cache && !cache_timer)
+		cache_timer = g_timeout_add_seconds(3, try_remove_cache, NULL);
 
 	g_free(server);
 }
@@ -2690,6 +2687,34 @@ static void append_domain(int index, const char *domain)
 	}
 }
 
+static void flush_requests(struct server_data *server)
+{
+	GSList *list;
+
+	list = request_list;
+	while (list) {
+		struct request_data *req = list->data;
+
+		list = list->next;
+
+		if (ns_resolv(server, req, req->request, req->name)) {
+			/*
+			 * A cached result was sent,
+			 * so the request can be released
+			 */
+			request_list =
+				g_slist_remove(request_list, req);
+			destroy_request_data(req);
+			continue;
+		}
+
+		if (req->timeout > 0)
+			g_source_remove(req->timeout);
+
+		req->timeout = g_timeout_add_seconds(5, request_timeout, req);
+	}
+}
+
 int __connman_dnsproxy_append(int index, const char *domain,
 							const char *server)
 {
@@ -2722,6 +2747,8 @@ int __connman_dnsproxy_append(int index, const char *domain,
 	if (!data)
 		return -EIO;
 
+	flush_requests(data);
+
 	return 0;
 }
 
@@ -2757,33 +2784,6 @@ int __connman_dnsproxy_remove(int index, const char *domain,
 	return 0;
 }
 
-void __connman_dnsproxy_flush(void)
-{
-	GSList *list;
-
-	list = request_list;
-	while (list) {
-		struct request_data *req = list->data;
-
-		list = list->next;
-
-		if (resolv(req, req->request, req->name)) {
-			/*
-			 * A cached result was sent,
-			 * so the request can be released
-			 */
-			request_list =
-				g_slist_remove(request_list, req);
-			destroy_request_data(req);
-			continue;
-		}
-
-		if (req->timeout > 0)
-			g_source_remove(req->timeout);
-		req->timeout = g_timeout_add_seconds(5, request_timeout, req);
-	}
-}
-
 static void dnsproxy_offline_mode(bool enabled)
 {
 	GSList *list;
@@ -3464,6 +3464,9 @@ static bool udp_listener_event(GIOChannel *channel, GIOCondition condition,
 		return true;
 	}
 
+	req->name = g_strdup(query);
+	req->request = g_malloc(len);
+	memcpy(req->request, buf, len);
 	req->timeout = g_timeout_add_seconds(5, request_timeout, req);
 	request_list = g_slist_append(request_list, req);
 
@@ -3864,6 +3867,16 @@ void __connman_dnsproxy_cleanup(void)
 {
 	DBG("");
 
+	if (cache_timer) {
+		g_source_remove(cache_timer);
+		cache_timer = 0;
+	}
+
+	if (cache) {
+		g_hash_table_destroy(cache);
+		cache = NULL;
+	}
+
 	connman_notifier_unregister(&dnsproxy_notifier);
 
 	g_hash_table_foreach(listener_table, remove_listener, NULL);
