@@ -2,8 +2,8 @@
  *
  *  ConnMan VPN daemon
  *
- *  Copyright (C) 2010  BMW Car IT GmbH. All rights reserved.
- *  Copyright (C) 2012  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2010,2013  BMW Car IT GmbH.
+ *  Copyright (C) 2012-2013  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -472,11 +472,11 @@ static void l2tp_died(struct connman_task *task, int exit_code, void *user_data)
 
 	vpn_died(task, exit_code, user_data);
 
-	conf_file = g_strdup_printf("/var/run/connman/connman-xl2tpd.conf");
+	conf_file = g_strdup_printf(VPN_STATEDIR "/connman-xl2tpd.conf");
 	unlink(conf_file);
 	g_free(conf_file);
 
-	conf_file = g_strdup_printf("/var/run/connman/connman-ppp-option.conf");
+	conf_file = g_strdup_printf(VPN_STATEDIR "/connman-ppp-option.conf");
 	unlink(conf_file);
 	g_free(conf_file);
 }
@@ -497,8 +497,9 @@ static void request_input_reply(DBusMessage *reply, void *user_data)
 
 	DBG("provider %p", l2tp_reply->provider);
 
-	if (dbus_message_get_type(reply) == DBUS_MESSAGE_TYPE_ERROR) {
-		error = dbus_message_get_error_name(reply);
+	if (!reply || dbus_message_get_type(reply) == DBUS_MESSAGE_TYPE_ERROR) {
+		if (reply)
+			error = dbus_message_get_error_name(reply);
 		goto done;
 	}
 
@@ -561,7 +562,8 @@ typedef void (* request_cb_t)(struct vpn_provider *provider,
 				const char *error, void *user_data);
 
 static int request_input(struct vpn_provider *provider,
-				request_cb_t callback, void *user_data)
+			request_cb_t callback, const char *dbus_sender,
+			void *user_data)
 {
 	DBusMessage *message;
 	const char *path, *agent_sender, *agent_path;
@@ -569,10 +571,11 @@ static int request_input(struct vpn_provider *provider,
 	DBusMessageIter dict;
 	struct request_input_reply *l2tp_reply;
 	int err;
+	void *agent;
 
-	connman_agent_get_info(&agent_sender, &agent_path);
-
-	if (!provider || !agent_path || !callback)
+	agent = connman_agent_get_info(dbus_sender, &agent_sender,
+							&agent_path);
+	if (!provider || !agent || !agent_path || !callback)
 		return -ESRCH;
 
 	message = dbus_message_new_method_call(agent_sender, agent_path,
@@ -607,7 +610,7 @@ static int request_input(struct vpn_provider *provider,
 
 	err = connman_agent_queue_message(provider, message,
 			connman_timeout_input_request(),
-			request_input_reply, l2tp_reply);
+			request_input_reply, l2tp_reply, agent);
 	if (err < 0 && err != -EBUSY) {
 		DBG("error %d sending agent request", err);
 		dbus_message_unref(message);
@@ -638,7 +641,7 @@ static int run_connect(struct vpn_provider *provider,
 
 	DBG("username %s password %p", username, password);
 
-	l2tp_name = g_strdup_printf("/var/run/connman/connman-xl2tpd.conf");
+	l2tp_name = g_strdup_printf(VPN_STATEDIR "/connman-xl2tpd.conf");
 
 	l2tp_fd = open(l2tp_name, O_RDWR|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR);
 	if (l2tp_fd < 0) {
@@ -648,7 +651,7 @@ static int run_connect(struct vpn_provider *provider,
 		goto done;
 	}
 
-	pppd_name = g_strdup_printf("/var/run/connman/connman-ppp-option.conf");
+	pppd_name = g_strdup_printf(VPN_STATEDIR "/connman-ppp-option.conf");
 
 	pppd_fd = open(pppd_name, O_RDWR|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR);
 	if (pppd_fd < 0) {
@@ -669,6 +672,8 @@ static int run_connect(struct vpn_provider *provider,
 
 	g_free(l2tp_name);
 	g_free(pppd_name);
+	close(l2tp_fd);
+	close(pppd_fd);
 
 	err = connman_task_run(task, l2tp_died, provider,
 				NULL, NULL, NULL);
@@ -716,7 +721,8 @@ static void request_input_cb(struct vpn_provider *provider,
 
 static int l2tp_connect(struct vpn_provider *provider,
 			struct connman_task *task, const char *if_name,
-			vpn_provider_connect_cb_t cb, void *user_data)
+			vpn_provider_connect_cb_t cb, const char *dbus_sender,
+			void *user_data)
 {
 	const char *username, *password;
 	int err;
@@ -744,7 +750,8 @@ static int l2tp_connect(struct vpn_provider *provider,
 		data->cb = cb;
 		data->user_data = user_data;
 
-		err = request_input(provider, request_input_cb, data);
+		err = request_input(provider, request_input_cb, dbus_sender,
+									data);
 		if (err != -EINPROGRESS) {
 			free_private_data(data);
 			goto done;
@@ -763,7 +770,7 @@ error:
 	return err;
 }
 
-static int l2tp_error_code(int exit_code)
+static int l2tp_error_code(struct vpn_provider *provider, int exit_code)
 {
 	switch (exit_code) {
 	case 1:
